// Dreieck.java                                     MM 2016
package Tools.Geometrie;

import java.awt.*;                                 // Color

/**
 * Dreieck, durch drei Punkte ABC eindeutig bestimmt.
 * @author Monika Meiler
 */
public class Dreieck
extends Zeichenobjekt implements Flaeche
{
/* ----------------------------------------------------- */
                                                // coreData
/**
 * Eckpunktkoordinate.
 */
  private Koordinate A;

/**
 * Eckpunktkoordinate.
 */
  private Koordinate B;

/**
 * Eckpunktkoordinate.
 */
  private Koordinate C;

/* ----------------------------------------------------- */
                                           // Konstruktoren
/**
 * Konstruktor,
 * setzt Eckpunkte und Farbe eines Dreiecks.
 * @param k1 Koordinate des 1. Eckpunktpunktes
 * @param k2 Koordinate des 2. Eckpunktpunktes
 * @param k3 Koordinate des 3. Eckpunktpunktes
 * @param farbe Dreiecksfarbe
 */
  public Dreieck
  ( Koordinate k1, Koordinate k2, Koordinate k3,
    Color farbe)
  {
    super( farbe);
    A = new Koordinate( k1);
    B = new Koordinate( k2);
    C = new Koordinate( k3);
  }

/**
 * Konstruktor,
 * setzt Eckpunkte und Farbe eines Dreiecks.
 * @param x1 x-Wert des 1. Eckpunktpunktes
 * @param y1 y-Wert des 1. Eckpunktpunktes
 * @param x2 x-Wert des 2. Eckpunktpunktes
 * @param y2 y-Wert des 2. Eckpunktpunktes
 * @param x3 x-Wert des 3. Eckpunktpunktes
 * @param y3 y-Wert des 3. Eckpunktpunktes
 * @param farbe Dreiecksfarbe
 */
  public Dreieck
  ( double x1, double y1, double x2, double y2,
    double x3, double y3, Color farbe)
  {
    this( new Koordinate( x1, y1),
          new Koordinate( x2, y2),
          new Koordinate( x3, y3), farbe);
  }

/* ----------------------------------------------------- */
                                        // Service-Methoden
/**
 * Berechnen des Flaecheninhalts eines Dreiecks.
 * @return Flaecheninhalt
 */
  public double getFlaeche()
  {
                            // Vektorprodukt (Kreuzprodukt)
    double ax, ay, bx, by;

    ax = B.getX() - A.getX();
    ay = B.getY() - A.getY();
    bx = C.getX() - A.getX();
    by = C.getY() - A.getY();

    return Math.abs( ax * by - bx * ay) / 2;
  }

/**
 * Berechnen des Umfangs eines Dreiecks.
 * @return Flaechenumfang
 */
  public double getUmfang()
  {
    double a, b, c;
      
    a = A.getAbstand( B);
    b = B.getAbstand( C);
    c = A.getAbstand( C);

    return a + b + c;
  }

/**
 * Grafische Darstellung, Zeichnen eines Dreiecks.
 */
  public void paint(Graphics g)
  {
    g.setColor( getColor());
    g.drawLine( (int)A.getX(), (int)A.getY(),
                (int)B.getX(), (int)B.getY());      
    g.drawLine( (int)B.getX(), (int)B.getY(),
                (int)C.getX(), (int)C.getY());      
    g.drawLine( (int)C.getX(), (int)C.getY(),
                (int)A.getX(), (int)A.getY());      
   }

/**
 * Beschreiben eines Dreiecks.
 */
  public String toString()
  {
    return "Dreieck: " + A + "-" + B + "-" + C;
  }

/* ----------------------------------------------------- */
/* ----------------------------------------------------- */
/**
 * Testprogramm:
 * rechtwinkliges Dreieck, Seitenlaengen 3, 4, 5.
 * Ausgabe: Flaeche = 6 Umfang = 12
 */
  public static void main( String[] args)
  {
    Color farbe = Color.blue;

    Dreieck dreieck0
    = new Dreieck( 1, 1, 5, 1, 1, 4, farbe);
    System.out.println( dreieck0);
    System.out.println
    ( "Flaeche = " + dreieck0.getFlaeche());
    System.out.println
    ( "Umfang  = " + dreieck0.getUmfang());

    Koordinate koord0 = new Koordinate( 1, 1);
    Koordinate koord1 = new Koordinate( 5, 1);
    Koordinate koord2 = new Koordinate( 1, 4);
    Dreieck dreieck1
    = new Dreieck( koord0, koord1, koord2, farbe);
    System.out.println( dreieck1);
    System.out.println
    ( "Flaeche = " + dreieck1.getFlaeche());
    System.out.println
    ( "Umfang  = " + dreieck1.getUmfang());
  }
}
