// Parallelogramm.java                              MM 2016
package Tools.Geometrie;

import java.awt.*;                                 // Color

/**
 * Parallelogramm, durch drei Punkte eindeutig bestimmt
 * ( drei Punkte beschreiben einen Winkel ABC
 *   eines Parallelogramms ABCD).
 * @author Monika Meiler
 */
public class Parallelogramm
extends Zeichenobjekt implements Flaeche
{
/* ----------------------------------------------------- */
                                                // coreData
/**
 * Eckpunktkoordinate.
 */
  private Koordinate A;

/**
 * Eckpunktkoordinate.
 */
  private Koordinate B;

/**
 * Eckpunktkoordinate.
 */
  private Koordinate C;

/**
 * Eckpunktkoordinate.
 */
  private Koordinate D;

/* ----------------------------------------------------- */
                                           // Konstruktoren
/**
 * Konstruktor,
 * setzt Eckpunkte und Farbe eines Parallelogramms.
 * @param k1 Koordinate des 1. Eckpunktpunktes
 * @param k2 Koordinate des 2. Eckpunktpunktes
 * @param k3 Koordinate des 3. Eckpunktpunktes
 * @param farbe Parallelogrammfarbe
 */
  public Parallelogramm
  ( Koordinate k1, Koordinate k2, Koordinate k3,
    Color farbe)
  {
    super( farbe);
    A = new Koordinate( k1);
    B = new Koordinate( k2);
    C = new Koordinate( k3);
    D = new Koordinate
             ( C.getX() + A.getX() - B.getX(),
               C.getY() + A.getY() - B.getY());
  }

/**
 * Konstruktor,
 * setzt Eckpunkte und Farbe eines Parallelogramms.
 * @param x1 x-Wert des 1. Eckpunktpunktes
 * @param y1 y-Wert des 1. Eckpunktpunktes
 * @param x2 x-Wert des 2. Eckpunktpunktes
 * @param y2 y-Wert des 2. Eckpunktpunktes
 * @param x3 x-Wert des 3. Eckpunktpunktes
 * @param y3 y-Wert des 3. Eckpunktpunktes
 * @param farbe Parallelogrammfarbe
 */
  public Parallelogramm
  ( double x1, double y1, double x2, double y2,
    double x3, double y3, Color farbe)
  {
    this( new Koordinate( x1, y1),
          new Koordinate( x2, y2),
          new Koordinate( x3, y3), farbe);
  }

/* ----------------------------------------------------- */
                                        // Service-Methoden
/**
 * Berechnen des Flaecheninhalts eines Parallelogramms.
 * @return Flaecheninhalt
 */
  public double getFlaeche()
  {
                            // Vektorprodukt (Kreuzprodukt)
    double ax, ay, bx, by;

    ax = B.getX() - A.getX();
    ay = B.getY() - A.getY();
    bx = C.getX() - A.getX();
    by = C.getY() - A.getY();

    return Math.abs( ax * by - bx * ay);
  }

/**
 * Berechnen des Umfangs eines Parallelogramms.
 * @return Flaechenumfang
 */
  public double getUmfang()
  {
    double a, b;
      
    a = A.getAbstand( B);
    b = B.getAbstand( C);

    return 2 * (a + b);
  }

/**
 * Grafische Darstellung, Zeichnen eines Parallelogramm.
 */
  public void paint( Graphics g)
  {
    g.setColor( getColor());
    g.drawLine( (int)A.getX(), (int)A.getY(),
                (int)B.getX(), (int)B.getY());      
    g.drawLine( (int)B.getX(), (int)B.getY(),
                (int)C.getX(), (int)C.getY());      
    g.drawLine( (int)C.getX(), (int)C.getY(),
                (int)D.getX(), (int)D.getY());      
    g.drawLine( (int)D.getX(), (int)D.getY(),
                (int)A.getX(), (int)A.getY());      
  }

/**
 * Beschreiben eines Parallelogramms.
 */
  public String toString()
  {
    return "Parallelogramm: " + 
           A + "-" + B + "-" + C + "-" + D;
  }

/* ----------------------------------------------------- */
/* ----------------------------------------------------- */
/**
 * Testprogramm:
 * Rechteck, Seitenlaengen 3 und 4.
 * Ausgabe: Flaeche = 12 Umfang = 14
 */
  public static void main( String[] args)
  {
    Color farbe = Color.blue;

    Parallelogramm rechteck0
    = new Parallelogramm( 1, 4, 1, 1, 5, 1, farbe);
    System.out.println( rechteck0);
    System.out.println
    ( "Flaeche = " + rechteck0.getFlaeche());
    System.out.println
    ( "Umfang  = " + rechteck0.getUmfang());

    Koordinate koord0 = new Koordinate( 1, 4);
    Koordinate koord1 = new Koordinate( 1, 1);
    Koordinate koord2 = new Koordinate( 5, 1);
    Parallelogramm rechteck1
    = new Parallelogramm( koord0, koord1, koord2, farbe);
    System.out.println( rechteck1);
    System.out.println
    ( "Flaeche = " + rechteck1.getFlaeche());
    System.out.println
    ( "Umfang  = " + rechteck1.getUmfang());
  }
}
