// Punkt.java                                       MM 2016
package Tools.Geometrie;

import java.awt.*;                                 // Color

/**
 * Farbiger Punkt im kartesischen Koordinatensystem.
 * @author Monika Meiler
 */
public class Punkt extends Zeichenobjekt
{
/* ----------------------------------------------------- */
                                                // coreData
/**
 * Punktkoordinate.
 */
  private Koordinate koordinate;

/* ----------------------------------------------------- */
                                           // Konstruktoren
/**
 * Konstruktor,
 * setzt Punktkoordinate und Punktfarbe.
 * @param k Koordinate
 * @param farbe Punktfarbe
 */
  public Punkt( Koordinate k, Color farbe)
  {
    super( farbe);
    koordinate = new Koordinate( k);
  }

/**
 * Konstruktor,
 * setzt x- und y-Wert der Punktkoordinate und -farbe.
 * @param x x-Koordinate
 * @param y y-Koordinate
 * @param farbe Punktfarbe
 */
  public Punkt( double x, double y, Color farbe)
  {
    this( new Koordinate( x, y), farbe);
  }

/* ----------------------------------------------------- */
                                         //getData-Methoden
/**
 * Lesen der Punktkoordinate.
 * @return Kopie der Punktkoordinate
 */
  public Koordinate getKoordinate()
  {
    return new Koordinate( koordinate);
  }

/* ----------------------------------------------------- */
                                        // Service-Methoden
/**
 * Grafische Darstellung,
 * zeichnet Punkt auf sich selbst.
 */
  public void paint( Graphics g)
  {
    int x = (int)koordinate.getX();
    int y = (int)koordinate.getY();

    g.setColor( getColor()); 
    g.drawLine( x, y, x, y);      
  }

/**
 * Abstand zu einem weiteren Punkt.
 * @param p Punkt
 * @return Abstand zwischen beiden Punkten
 */
  public double getAbstand( Punkt p)
  {
     return koordinate.getAbstand( p.getKoordinate());
  }

/**
 * Beschreiben eines Punktes.
 */
  public String toString()
  {
    return "Punkt: " + koordinate; 
  }

/* ----------------------------------------------------- */
/* ----------------------------------------------------- */
/**
 * Testprogramm:
 * pythagoreisches Zahlentripel(3, 4, 5).
 * Ausgabe: Abstand 5.0
 */
  public static void main( String[] args)
  {
    Color farbe = Color.blue;

    Punkt punkt0 = new Punkt( 1, 2, farbe);

    Koordinate koord1 = new Koordinate( 5, 5);
    Punkt punkt1 = new Punkt( koord1, farbe);

    System.out.println( punkt0);
    System.out.println( punkt1);

    System.out.println
    ( "Abstand " + punkt0.getAbstand( punkt1));
  }
}
